<?php

if (!defined("PHORUM")) return;

/**
 *    The other Phorum code does not care how the messages are stored.
 *    The only requirement is that they are returned from these functions
 *    in the right way.  This means each database can use as many or as
 *    few tables as it likes.  It can store the fields anyway it wants.
 *    The only thing to worry about is the table_prefix for the tables.
 *    all tables for a Phorum install should be prefixed with the
 *    table_prefix that will be entered in include/db/config.php.  This
 *    will allow multiple Phorum installations to use the same database.
 */

/*
 * TO DO
 *
 * Full text search on message_table
 * uncomment call to phorum_db_update_settings in phorum_db_create_tables
 * uncomment call to phorum_db_add_forum       in phorum_db_create_tables
 *
 */

/**
 * These are the table names used for this database system.
 */

$PHORUM["settings_table"]           = "{$PHORUM['DBCONFIG']['table_prefix']}_settings";
$PHORUM["forums_table"]             = "{$PHORUM['DBCONFIG']['table_prefix']}_forums";
$PHORUM["message_table"]            = "{$PHORUM['DBCONFIG']['table_prefix']}_messages";
$PHORUM["user_table"]               = "{$PHORUM['DBCONFIG']['table_prefix']}_users";
$PHORUM["user_permissions_table"]   = "{$PHORUM['DBCONFIG']['table_prefix']}_user_permissions";
$PHORUM["user_newflags_table"]      = "{$PHORUM['DBCONFIG']['table_prefix']}_user_newflags";
$PHORUM["subscribers_table"]        = "{$PHORUM['DBCONFIG']['table_prefix']}_subscribers";
$PHORUM["groups_table"]             = "{$PHORUM['DBCONFIG']['table_prefix']}_groups";
$PHORUM["forum_group_xref_table"]   = "{$PHORUM['DBCONFIG']['table_prefix']}_forum_group_xref";
$PHORUM["user_group_xref_table"]    = "{$PHORUM['DBCONFIG']['table_prefix']}_user_group_xref";
$PHORUM["files_table"]              = "{$PHORUM['DBCONFIG']['table_prefix']}_files";
$PHORUM["message_files_xref_table"] = "{$PHORUM['DBCONFIG']['table_prefix']}_message_files_xref";
$PHORUM["banlist_table"]            = "{$PHORUM['DBCONFIG']['table_prefix']}_banlists";
$PHORUM["private_message_table"]    = "{$PHORUM['DBCONFIG']['table_prefix']}_private_messages";

/**
 * These are the names of the indexes for the above table.
 * There is no mention of how to create INDEXES in the ANSI
 * standard.  But just about every database I know allows
 * for CREATE INDEX.
 *
 * Much safer and better to keep the indexes separate.
 *
 * I suggest this be copied to the MySQL module.
 * See also:
 *      http://sql-info.de/mysql/gotchas.html
 *      http://sql-info.de/postgresql/postgres-gotchas.html
 */

$PHORUM_IDX["forums_table_active"]                = $PHORUM["forums_table"]            . "_active_idx";
$PHORUM_IDX["forums_table_name"]                  = $PHORUM["forums_table"]            . "_name_idx";
$PHORUM_IDX["forums_table_group_id"]              = $PHORUM["forums_table"]            . "_group_idx";

$PHORUM_IDX["message_table_message_id"]           = $PHORUM["message_table"]           . "_message_id_idx";
$PHORUM_IDX["message_table_thread_message"]       = $PHORUM["message_table"]           . "_thread_message_idx";
$PHORUM_IDX["message_table_thread_forum"]         = $PHORUM["message_table"]           . "_thread_forum_idx";
$PHORUM_IDX["message_table_list_page_flat"]       = $PHORUM["message_table"]           . "_list_page_flat_idx";
$PHORUM_IDX["message_table_special_threads"]      = $PHORUM["message_table"]           . "_special_threads_idx";
$PHORUM_IDX["message_table_list_page_float"]      = $PHORUM["message_table"]           . "_list_page_float_idx";

$PHORUM_IDX["subscribers_table_forum_id"]         = $PHORUM["subscribers_table"]       . "_forum_id_idx";

$PHORUM_IDX["user_permissions_table_forum_id"]    = $PHORUM["user_permissions_table"]  . "_forum_id_idx";

$PHORUM_IDX["user_table_username"]                = $PHORUM["user_table"]              . "_username_idx";
$PHORUM_IDX["user_table_active"]                  = $PHORUM["user_table"]              . "_active_idx";
$PHORUM_IDX["user_table_userpass"]                = $PHORUM["user_table"]              . "_userpass_idx";

$PHORUM_IDX["files_table_add_datetime"]           = $PHORUM["files_table"]             . "_add_datetime_idx";
$PHORUM_IDX["files_table_message_id"]             = $PHORUM["files_table"]             . "_message_id_idx";

$PHORUM_IDX["banlist_table_forum_id"]             = $PHORUM["banlist_table"]           . "_forum_id_idx";

$PHORUM_IDX["private_message_table_to_user_id"]   = $PHORUM["private_message_table"]   . "_to_user_id_idx";
$PHORUM_IDX["private_message_table_from_user_id"] = $PHORUM["private_message_table"]   . "_from_user_id_idx";
$PHORUM_IDX["private_message_table_to_del_flag"]  = $PHORUM["private_message_table"]   . "_to_del_flag_idx";


/**
 * This function executes a query to select messages from the database
 * and returns an array.  The main Phorum code handles actually sorting
 * the threads into a threaded list if needed.
 *
 * NOTE: ALL dates should be returned as Unix timestamps
 */

function phorum_db_get_thread_list($offset)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $table = $PHORUM["message_table"];

    $arr = array();
    $keyids = array();

    if($PHORUM["threaded_list"]){

        $keyfield="thread";
        $sortorder = "sort, thread desc, parent_id, message_id";
        $limit=$PHORUM['list_length'];
        
        $offset_option="thread > 0 and";
        
        if(empty($offset)){

            // get the announcements and stickies
            $sql="select $keyfield as keyid from $table where sort=0 or (sort=1 and forum_id={$PHORUM['forum_id']}) order by sort, thread desc";
            
            $res = pg_query($conn, $sql);
            
            if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
            
            $rows=pg_numrows($res);

            if ($rows > 0){

                while($rec = pg_fetch_array($res)){
                    if(!empty($rec["keyid"])) $keyids[$rec["keyid"]] = $rec["keyid"];
                }
            
                $limit-=$rows;
    
            }
            
        } else {
            $offset_option="thread < $offset and";
        }                        
        
        $sql = "select
                    thread as keyid
                from
                    $table
                where
                    $offset_option
                    forum_id in ({$PHORUM['forum_id']}, 0) and
                    parent_id>=0 and
                    status>0 and
                    sort>1
                order by
                    thread desc
                limit $limit";


    } else {

        $sortfield = ($PHORUM["float_to_top"]) ? "modifystamp" : "thread";
        $start = $offset * $PHORUM["list_length"];
        $sortorder = "sort, $sortfield desc, message_id";
        $keyfield = "message_id";
        
        $limit=$PHORUM['list_length'];

        // get the announcements and stickies
        $sql="select $keyfield as keyid from $table where sort=0 or (parent_id=0 and sort=1 and forum_id={$PHORUM['forum_id']}) order by sort, $sortfield desc";

        $res = pg_query($conn, $sql);

        $rows = pg_numrows($res);       
        
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    
        if(empty($offset)){
            if (pg_numrows($res) > 0){
                while($rec = pg_fetch_array($res)){
                    if(!empty($rec["keyid"])) $keyids[$rec["keyid"]] = $rec["keyid"];
                }
            
                $limit-=count($keyids);
    
            }
        } else {
            $start-=$rows;
        }
        
        $sql = "select
                    thread as keyid
                from
                    $table
                where
                    $sortfield>0 and
                    forum_id in ({$PHORUM['forum_id']}, 0) and
                    parent_id=0 and
                    status>0 and
                    sort>1
                order by
                    $sortfield desc
                limit $start, $limit";

    }

    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    if (pg_numrows($res) > 0){
        while($rec = pg_fetch_array($res)){
            $keyids[$rec["keyid"]] = $rec["keyid"];
        }

    }
    
    if(count($keyids)>0){

        $sql = "select
                $table.author,
                $table.datestamp,
                $table.email,
                $table.message_id,
                $table.meta,
                $table.moderator_post,
                $table.modifystamp,
                $table.parent_id,
                $table.sort,
                $table.status,
                $table.subject,
                $table.thread,
                $table.thread_count,
                $table.user_id
              from
                $table
              where
                $keyfield in (" . implode(",", $keyids) . ")
              order by
                $sortorder";

        $res = pg_query($conn, $sql);

        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

        while ($rec = pg_fetch_array($res)){
            $arr[$rec["message_id"]] = $rec;
            $arr[$rec["message_id"]]["meta"] = array();
            if(!empty($rec["meta"])){
                $arr[$rec["message_id"]]["meta"] = unserialize($rec["meta"]);
            }
        }

    }

    return $arr;
}


/**
 * This function executes a query to find the previous pages starting thread
 * for threaded mode list pages.
 */

function phorum_db_get_previous_start_thread($thread)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $table = $PHORUM["message_table"];

    $sql="select thread from $table where thread > $thread and forum_id in ({$PHORUM['forum_id']}, 0) order by thread limit {$PHORUM['list_length']}";    
  
    $res = pg_query($conn, $sql);

    $rows=pg_numrows($res);

    if($rows>0){
        $new_thread=pg_fetch_result($res, $rows-1, "thread") + 1;
    } else {
        $new_thread=0;
    }    

    return $new_thread;    
}

/**
 * This function executes a query to select messages from the database
 * and returns an array.  The main Phorum code handles actually sorting
 * the threads into a threaded list if needed.
 *
 * NOTE: ALL dates should be returned as Unix timestamps
 */

function phorum_db_get_unapproved_list()
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $table = $PHORUM["message_table"];

    $arr = array();

    $sql = "select
            $table.author,
            $table.datestamp,
            $table.email,
            $table.message_id,
            $table.meta,
            $table.moderator_post,
            $table.modifystamp,
            $table.parent_id,
            $table.sort,
            $table.status,
            $table.subject,
            $table.thread,
            $table.thread_count,
            $table.user_id
          from
            $table
          where
            status < 0 and forum_id = {$PHORUM['forum_id']}
          order by
            message_id desc";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    while ($rec = pg_fetch_array($res)){
        $arr[$rec["message_id"]] = $rec;
        $arr[$rec["message_id"]]["meta"] = array();
        if(!empty($rec["meta"])){
            $arr[$rec["message_id"]]["meta"] = unserialize($rec["meta"]);
        }
    }

    return $arr;
}


/**
 * This function posts a message to the tables.
 * The message is passed by reference and message_id and thread are filled
 */

function phorum_db_post_message(&$message){
    $PHORUM = $GLOBALS["PHORUM"];
    $table = $PHORUM["message_table"];

    $conn = phorum_db_postgresql_connect();

    $success = false;

    foreach($message as $key => $value){
        if (is_numeric($value)){
            $message[$key] = (int)$value;
        } elseif(is_array($value)) {
            $message[$key] = pg_escape_string(serialize($value));
        } else{
            $message[$key] = pg_escape_string($value);
        }
    }

    $NOW = time();

    // duplicate-check
    if($PHORUM['check_duplicate']) {
        // we check for dupes in that number of minutes
        $check_minutes=60;
        $check_timestamp =$NOW - ($check_minutes*60);
        // check_query
        $chk_query="SELECT message_id FROM $table WHERE forum_id = {$message['forum_id']} AND author='{$message['author']}' AND subject='{$message['subject']}' AND body='{$message['body']}' AND datestamp > $check_timestamp";
        $res = pg_query($conn, $chk_query);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
        if(pg_numrows($res))
            return 0;
    }

    if(isset($message['meta']))
        $metaval=",meta='{$message['meta']}'";
    else
        $metaval="";


    $sql = "Insert into $table set
            forum_id = {$message['forum_id']},
            datestamp=$NOW,
            thread={$message['thread']},
            parent_id={$message['parent_id']},
            author='{$message['author']}',
            subject='{$message['subject']}',
            email='{$message['email']}',
            ip='{$message['ip']}',
            user_id={$message['user_id']},
            moderator_post={$message['moderator_post']},
            status={$message['status']},
            sort={$message['sort']},
            msgid='{$message['msgid']}',
            body='{$message['body']}'
            $metaval";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    if ($res){
        $message["message_id"] = phorum_db_pg_insertid($conn, "$table" . "id_seq");

        if(!empty($message["message_id"])){

            $message["datestamp"]=$NOW;

            if ($message["thread"] == 0){
                $message["thread"] = $message["message_id"];
                $sql = "update $table set thread={$message['message_id']} where message_id={$message['message_id']}";
                $res = pg_query($conn, $sql);
                if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
            }

            if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

            $success = true;
            // some data for later use, i.e. email-notification
            $GLOBALS['PHORUM']['post_returns']['message_id']=$message["message_id"];
            $GLOBALS['PHORUM']['post_returns']['thread_id']=$message["thread"];            
        }
    }

    return $success;
}

/**
 * This function deletes messages from the messages table.
 *
 * @param message $ _id the id of the message which should be deleted
 * mode the mode of deletion, 0 for reconnecting the children, 1 for deleting the children
 */

function phorum_db_delete_message($message_id, $mode = 0)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    // lock the table so we don't leave orphans.
    mysql_query("LOCK TABLES {$PHORUM['message_table']} WRITE", $conn);

    if($mode == PHORUM_DELETE_TREE){
        $mids = phorum_db_get_messagetree($message_id);
    }else{
        $mids = $message_id;
    }

    $threadset = 0;
    // get the parents of the message to delete.
    $sql = "select forum_id, message_id, thread, parent_id from {$PHORUM['message_table']} where message_id = $message_id ";
    $res = pg_query($conn, $sql);
    $rec = pg_fetch_array($res);

    $thread = $rec['thread'];
    if($thread == $message_id && $mode == PHORUM_DELETE_TREE){
        $threadset = 1;
    }else{
        $threadset = 0;
    }

    if($mode == PHORUM_DELETE_MESSAGE){
        $count = 1;
        // change the children to point to their parent's parent
        $sql = "update {$PHORUM['message_table']} set parent_id=$rec[parent_id] where parent_id=$rec[message_id]";
        pg_query($conn, $sql);
    }else{
        $count = count(explode(",", $mids));
    }

    // delete the messages
    $sql = "delete from {$PHORUM['message_table']} where message_id in ($mids)";
    pg_query($conn, $sql);

    // clear the lock
    mysql_query("UNLOCK TABLES", $conn);

    // it kind of sucks to have this here, but it is the best way
    // to ensure that it gets done if stuff is deleted.
    // leave this include here, it needs to be conditional
    include_once("./include/thread_info.php");
    phorum_update_thread_info($thread);

    // this function will be slow with a lot of messages.
    phorum_db_update_forum_stats(true);

    return explode(",", $mids);
}

/**
 * gets all attached messages to a message
 *
 * @param id $ id of the message
 */
function phorum_db_get_messagetree($id){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $sql = "Select message_id from {$PHORUM['message_table']} where parent_id=$id";

    $res = pg_query($conn, $sql);

    $tree = "$id";

    while($rec = pg_fetch_row($res, 0)){
        $tree .= "," . phorum_db_get_messagetree($rec[0]);
    }

    return $tree;
}

/**
 * This function updates the message given in the $message array for
 * the row with the given message id.  It returns non 0 on success.
 */

function phorum_db_update_message($message_id, $message){
    $PHORUM = $GLOBALS["PHORUM"];

    if (count($message) > 0){
        $conn = phorum_db_postgresql_connect();

        foreach($message as $field => $value){
            if (is_numeric($value)){
                $fields[] = "$field=$value";
            }elseif (is_array($value)){
                $fields[] = "$field='".pg_escape_string(serialize($value))."'";
            }else{
                $value = pg_escape_string($value);
                $fields[] = "$field='$value'";
            }
        }

        $sql = "update {$PHORUM['message_table']} set " . implode(", ", $fields) . " where message_id=$message_id";
        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

        return ($res > 0) ? true : false;
    }else{
        trigger_error("\$message cannot be empty in phorum_update_message()", E_USER_ERROR);
    }
}


/**
 * This function executes a query to get the row with the given message
 * id and returns the message in an array.
 */

function phorum_db_get_message($message_id){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $forum_id_check = "";
    if (!empty($PHORUM["forum_id"])){
        $forum_id_check = "(forum_id = {$PHORUM['forum_id']} OR forum_id=0) and";
    }

    $sql = "select {$PHORUM['message_table']}.* from {$PHORUM['message_table']} where $forum_id_check message_id=$message_id";

    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $rec=array();

    if(pg_numrows($res)){

        $rec = pg_fetch_array($res);
    
        // convert meta field
        if(empty($rec["meta"])){
            $rec["meta"]=array();
        } else {
            $rec["meta"]=unserialize($rec["meta"]);
        }
    }

    return $rec;
}

/**
 * This function executes a query to get the rows with the given thread
 * id and returns an array of the message.
 */

function phorum_db_get_messages($thread,$page=0){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $forum_id_check = "";
    if (!empty($PHORUM["forum_id"])){
        $forum_id_check = "(forum_id = {$PHORUM['forum_id']} OR forum_id=0) and";
    }

    // are we really allowed to show this thread/message?
    $approvedval = "";
    if(!phorum_user_access_allowed(PHORUM_USER_ALLOW_MODERATE_MESSAGES) && $PHORUM["moderation"] == PHORUM_MODERATE_ON) {
        $approvedval="AND {$PHORUM['message_table']}.status > 0 ";
    }

    if($page > 0) {
           $start=$PHORUM["read_length"]*($page-1);
           $sql = "select {$PHORUM['message_table']}.* from {$PHORUM['message_table']} where $forum_id_check thread=$thread $approvedval order by message_id LIMIT $start,".$PHORUM["read_length"];
    } else {
           $sql = "select {$PHORUM['message_table']}.* from {$PHORUM['message_table']} where $forum_id_check thread=$thread $approvedval order by message_id";
    }

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $arr = array();

    while ($rec = pg_fetch_array($res)){
        $arr[$rec["message_id"]] = $rec;

        // convert meta field
        if(empty($rec["meta"])){
            $arr[$rec["message_id"]]["meta"]=array();
        } else {
            $arr[$rec["message_id"]]["meta"]=unserialize($rec["meta"]);
        }
        if(empty($arr['users'])) $arr['users']=array();
        if($rec["user_id"]){
            $arr['users'][]=$rec["user_id"];
        }

    }

    if(count($arr) && $page != 0) {
        // selecting the thread-starter
        $sql = "select {$PHORUM['message_table']}.* from {$PHORUM['message_table']} where $forum_id_check message_id=$thread $approvedval";
        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
		if(pg_numrows($res) > 0) {
	        $rec = pg_fetch_array($res);
	        $arr[$rec["message_id"]] = $rec;
	        $arr[$rec["message_id"]]["meta"]=unserialize($rec["meta"]);
		}
    }
    return $arr;
}

/**
 * this function returns the index of a message in a thread
 */
function phorum_db_get_message_index($thread=0,$message_id=0) {
    $PHORUM = $GLOBALS["PHORUM"];

    // check for valid values
    if(empty($message_id) || empty($message_id)) {
        return 0;
    }

    $approvedval="";
    $forum_id_check="";

    $conn = phorum_db_postgresql_connect();

    if (!empty($PHORUM["forum_id"])){
        $forum_id_check = "(forum_id = {$PHORUM['forum_id']} OR forum_id=0) AND";
    }

    if(!phorum_user_access_allowed(PHORUM_USER_ALLOW_MODERATE_MESSAGES) && $PHORUM["moderation"] == PHORUM_MODERATE_ON) {
        $approvedval="AND {$PHORUM['message_table']}.status > 0 ";
    }

    $sql = "select count(*) as msg_index from {$PHORUM['message_table']} where $forum_id_check thread=$thread $approvedval AND message_id <= $message_id order by message_id";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $rec = pg_fetch_array($res);

    return $rec['msg_index'];
}

/**
 * This function searches the database for the supplied search
 * criteria and returns an array with two elements.  One is the count
 * of total messages that matched, the second is an array of the
 * messages from the results based on the $start (0 base) given and
 * the $length given.
 */

function phorum_db_search($search, $offset, $length, $match_type, $match_date, $match_forum, $body, $author, $subject){
    $PHORUM = $GLOBALS["PHORUM"];

    $start = $offset * $PHORUM["list_length"];

    $arr = array("count" => 0, "rows" => array());

    $conn = phorum_db_postgresql_connect();

    $sql = "select count(*) as count from {$PHORUM['message_table']}";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $total_messages = pg_fetch_result($res, 0, "count");

    if($match_type=="PHRASE"){
        $terms = $search;
    } else {
        $terms = preg_split("/\s+/", $search);
    }
        
    foreach($terms as $term){
        $fields=array();
        if($body) $fields[]="body like '%$term%'";
        if($author) $fields[]="author like '%$term%'";
        if($subject) $fields[]="subject like '%$term%'";
        if($fields){
            $clause[] = "( ".implode(" or ", $fields)." )";
        } else {
            return $arr;
        }
    }

    $conj = ($match_type=="ALL") ? "and" : "or";

    $sql = "select message_id from {$PHORUM['message_table']} where " . implode(" $conj ", $clause);

    if($match_date>0){
        $ts=time()-86400*$match_date;
        $sql.=" and datestamp>=$ts";
    }

    if($PHORUM['forum_id']!=0 && $match_forum!="ALL"){
        $sql.=" and forum_id={$PHORUM['forum_id']}";
    } else {
        $sql.=" and forum_id<>0";
    }

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    if (pg_numrows($res)){
        while ($rec = pg_fetch_row($res, 0)){
            $total_ids[] = $rec[0];
        }
        // don't worry about how many ids we have here.
        $sql = "select message_id from {$PHORUM['message_table']} where message_id in (" . implode(",", $total_ids) . ") order by datestamp desc";

        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

        if (pg_result_seek($res, $start)){
            $ids = array();

            while (($rec = pg_fetch_row($res, 0)) && count($ids) < $length){
                $ids[] = $rec[0];
            }
            // don't worry about how many ids we have here.
            $sql = "select {$PHORUM['message_table']}.* from {$PHORUM['message_table']} where message_id in (" . implode(",", $ids) . ") order by datestamp desc";

            $res = pg_query($conn, $sql);
            if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

            $rows = array();

            while ($rec = pg_fetch_array($res)){
                $rows[$rec["message_id"]] = $rec;
            }

            $arr = array("count" => count($total_ids), "rows" => $rows, "total" => $total_messages);
        }
    }

    return $arr;
}

/**
 * This function returns the closest thread that is greater than $thread
 */

function phorum_db_get_newer_thread($key){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $keyfield = ($PHORUM["float_to_top"]) ? "modifystamp" : "thread";
	
    // are we really allowed to show this thread/message?
    $approvedval = "";
    if(!phorum_user_access_allowed(PHORUM_USER_ALLOW_MODERATE_MESSAGES) && $PHORUM["moderation"] == PHORUM_MODERATE_ON) {
        $approvedval="AND {$PHORUM['message_table']}.status > 0 ";
    }	

    $sql = "select thread from {$PHORUM['message_table']} where forum_id={$PHORUM['forum_id']} and $keyfield>$key $approvedval order by $keyfield limit 1";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    return (pg_numrows($res)) ? pg_fetch_result($res, 0, "thread") : 0;
}

/**
 * This function returns the closest thread that is less than $thread
 */

function phorum_db_get_older_thread($key){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $keyfield = ($PHORUM["float_to_top"]) ? "modifystamp" : "thread";
    // are we really allowed to show this thread/message?
    $approvedval = "";
    if(!phorum_user_access_allowed(PHORUM_USER_ALLOW_MODERATE_MESSAGES) && $PHORUM["moderation"] == PHORUM_MODERATE_ON) {
        $approvedval="AND {$PHORUM['message_table']}.status > 0 ";
    }	

    $sql = "select thread from {$PHORUM['message_table']} where forum_id={$PHORUM['forum_id']} and $keyfield<$key $approvedval order by $keyfield desc limit 1";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    return (pg_numrows($res)) ? pg_fetch_result($res, 0, "thread") : 0;
}

/**
 * This function executes a query to get bad items of type $type and
 * returns an array of the results.
 */

function phorum_db_load_settings(){
    global $PHORUM;

    $conn = phorum_db_postgresql_connect();

    $sql = "select * from {$PHORUM['settings_table']}";

    $res = pg_query($conn, $sql);
    if (($err = pg_last_error()) && !defined("PHORUM_ADMIN")) phorum_db_pg_last_error("$err: $sql");

    if (empty($err)){
        while ($rec = pg_fetch_array($res)){
            if ($rec["type"] == "V"){
                if ($rec["data"] == 'true'){
                    $val = true;
                }elseif ($rec["data"] == 'false'){
                    $val = false;
                }elseif (is_numeric($rec["data"])){
                    $val = $rec["data"];
                }else{
                    $val = "$rec[data]";
                }
            }else{
                $val = unserialize($rec["data"]);
            }

            $PHORUM[$rec['name']]=$val;
            $PHORUM['SETTINGS'][$rec['name']]=$val;
        }
    }
}

/**
 * This function executes a query to get bad items of type $type and
 * returns an array of the results.
 */

function phorum_db_update_settings($settings){
    global $PHORUM;

    if (count($settings) > 0){
        $conn = phorum_db_postgresql_connect();

        foreach($settings as $field => $value){
            if (is_numeric($value)){
                $type = 'V';
            }elseif (is_string($value)){
                $value = pg_escape_string($value);
                $type = 'V';
            }else{
                $value = pg_escape_string(serialize($value));
                $type = 'S';
            }

            $sql = "replace into {$PHORUM['settings_table']} set data='$value', type='$type', name='$field'";
            $res = pg_query($conn, $sql);
            if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
        }

        return ($res > 0) ? true : false;
    }else{
        trigger_error("\$settings cannot be empty in phorum_db_update_settings()", E_USER_ERROR);
    }
}

/**
 * This function executes a query to select all forum data from
 * the database for a flat/collapsed display and returns the data in
 * an array.
 */

function phorum_db_get_forums($forum_ids = 0, $parent_id = null){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    if (is_array($forum_ids)) $forum_ids = implode(",", $forum_ids);

    $sql = "select * from {$PHORUM['forums_table']}";
    if ($forum_ids){
        $sql .= " where forum_id in ($forum_ids)";
    }elseif (func_num_args() > 1){
        $sql .= " where parent_id = $parent_id";
    }

    $sql .= " order by display_order ASC, name";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $forums = array();

    while ($row = pg_fetch_array($res)){
        $forums[$row["forum_id"]] = $row;
    }

    return $forums;
}

/**
 * This function updates the forums stats.  If refresh is true, it pulls the
 * numbers from the table.
 */

function phorum_db_update_forum_stats($refresh=false, $msg_count_change=1, $timestamp=0)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    // we do this if there aren't many rows for this forum.
    if($refresh || $PHORUM["message_count"]<500000){

        $sql = "select count(*) as message_count from {$PHORUM['message_table']} where thread>0 and status in (".PHORUM_STATUS_CLOSED.", ".PHORUM_STATUS_APPROVED.") and forum_id={$PHORUM['forum_id']}";

        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

        $message_count = pg_fetch_result($res, 0, "message_count");

        $sql = "select max(modifystamp) as last_post_time from {$PHORUM['message_table']} where modifystamp>0 and status in (".PHORUM_STATUS_CLOSED.", ".PHORUM_STATUS_APPROVED.") and forum_id={$PHORUM['forum_id']}";

        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

        $last_post_time = pg_fetch_result($res, 0, "last_post_time");

    } else {
        
        // incrementing the message-count        
        $message_count="message_count+$msg_count_change";
        
        // checking the last-post-time
        $last_post_time = (empty($timestamp)) ? time() : $timestamp; 
        
        // don't set it to a lower value
        if($PHORUM["last_post_time"]>=$last_post_time) {
            $last_post_time=$row[0];
        }
    }

    // get the thread count
    // no good way to shortcut this one
    $sql = "select count(*) as message_count from {$PHORUM['message_table']} where thread>0 and parent_id=0 and status in (".PHORUM_STATUS_CLOSED.", ".PHORUM_STATUS_APPROVED.") and forum_id={$PHORUM['forum_id']}";
    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    list($thread_count) = pg_fetch_row($res, 0);        



    $last_post_time=isset($last_post_time) ? $last_post_time : 0;

    $sql = "update {$PHORUM['forums_table']} set thread_count=$thread_count, message_count=$message_count, last_post_time=$last_post_time where forum_id={$PHORUM['forum_id']}";
    pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

}

/**
 * actually moves a thread to the given forum
 */
function phorum_db_move_thread($thread_id, $toforum)
{
    $PHORUM = $GLOBALS["PHORUM"];

    if($toforum > 0 && $thread_id > 0){
        $conn = phorum_db_postgresql_connect();

        // just changing the forum-id, simple isn't it?
        $sql = "UPDATE {$PHORUM['message_table']} SET forum_id=$toforum where thread=$thread_id";

        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
        
        // we need to update the number of posts in the current forum
        phorum_db_update_forum_stats(true);
        
        // and of the new forum
        $old_id=$GLOBALS["PHORUM"]["forum_id"];
        $GLOBALS["PHORUM"]["forum_id"]=$toforum;
        phorum_db_update_forum_stats(true);
        $GLOBALS["PHORUM"]["forum_id"]=$old_id;

    }
}

/**
 * closes the given thread
 */
function phorum_db_close_thread($thread_id){
    $PHORUM = $GLOBALS["PHORUM"];

    if($thread_id > 0){
        $conn = phorum_db_postgresql_connect();

        $sql = "UPDATE {$PHORUM['message_table']} SET status=" . PHORUM_STATUS_CLOSED . " where thread=$thread_id";

        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    }
}

/**
 * closes the given thread
 */
function phorum_db_reopen_thread($thread_id){
    $PHORUM = $GLOBALS["PHORUM"];

    if($thread_id > 0){
        $conn = phorum_db_postgresql_connect();

        $sql = "UPDATE {$PHORUM['message_table']} SET status=" . PHORUM_STATUS_APPROVED . " where thread=$thread_id";

        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    }
}

/**
 * This function executes a query to insert a forum into the forums
 * table and returns the forums id on success or 0 on failure.
 */

function phorum_db_add_forum($forum){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    foreach($forum as $key => $value){
        if (is_numeric($value)){
            $value = (int)$value;
            $fields[] = "$key=$value";
        }else{
            $value = pg_escape_string($value);
            $fields[] = "$key='$value'";
        }
    }

    $sql = "insert into {$PHORUM['forums_table']} set " . implode(", ", $fields);

echo "<pre>$sql</pre>";
    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $forum_id = 0;

    if ($res){
        $forum_id = mysql_insert_id($conn);
    }

    return $forum_id;
}

/**
 * This function executes a query to remove a forum from the forums
 * table and its messages.
 */

function phorum_db_drop_forum($forum_id){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $sql = "delete from {$PHORUM['forums_table']} where forum_id=$forum_id";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $sql = "delete from {$PHORUM['message_table']} where forum_id=$forum_id";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
}

/**
 * This function executes a query to remove a folder from the forums
 * table and change the parent of its children.
 */

function phorum_db_drop_folder($forum_id){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $sql = "select parent_id from {$PHORUM['forums_table']} where forum_id=$forum_id";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $new_parent_id = pg_fetch_result($res, 0, "parent_id");

    $sql = "update {$PHORUM['forums_table']} set parent_id=$new_parent_id where parent_id=$forum_id";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $sql = "delete from {$PHORUM['forums_table']} where forum_id=$forum_id";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
}

/**
 * This function executes a query to update a forum in the forums
 * table and returns non zero on success or 0 on failure.
 */

function phorum_db_update_forum($forum){
    $PHORUM = $GLOBALS["PHORUM"];

    $res = 0;

    if (!empty($forum["forum_id"])){
    
        // this way we can also update multiple forums at once
        if(is_array($forum["forum_id"])) {
            $forumwhere="forum_id IN (".implode(",",$forum["forum_id"]).")";        
        } else {
            $forumwhere="forum_id=".$forum["forum_id"];
        }
    
        unset($forum["forum_id"]);

        $conn = phorum_db_postgresql_connect();

        foreach($forum as $key => $value){
            if (is_numeric($value)){
                $value = (int)$value;
                $fields[] = "$key=$value";
            }else{
                $value = pg_escape_string($value);
                $fields[] = "$key='$value'";
            }
        }

        $sql = "update {$PHORUM['forums_table']} set " . implode(", ", $fields) . " where $forumwhere";

        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    }else{
        trigger_error("\$forum[forum_id] cannot be empty in phorum_update_forum()", E_USER_ERROR);
    }

    return $res;
}

/**
*
*/

function phorum_db_get_groups($group_id=0)
{
    $PHORUM = $GLOBALS["PHORUM"];
    $conn = phorum_db_postgresql_connect();

    settype($group_id, "integer");

    $sql="select * from {$PHORUM['groups_table']}";
    if($group_id!=0) $sql.=" where group_id=$group_id";

    $res = pg_query($conn, $sql);
    
    $groups=array();
    while($rec=pg_fetch_array($res)){

        $groups[$rec["group_id"]]=$rec;
        $groups[$rec["group_id"]]["permissions"]=array();
    }

    $sql="select * from {$PHORUM['forum_group_xref_table']}";
    if($group_id!=0) $sql.=" where group_id=$group_id";

    $res = pg_query($conn, $sql);

    while($rec=pg_fetch_array($res)){

        $groups[$rec["group_id"]]["permissions"][$rec["forum_id"]]=$rec["permission"];

    }

    return $groups;

}


/**
*
*/

function phorum_db_save_group($group)
{
    $PHORUM = $GLOBALS["PHORUM"];
    $conn = phorum_db_postgresql_connect();

    $ret=false;

    if(isset($group["name"])){
        $sql="update {$PHORUM['groups_table']} set name='{$group['name']}' where group_id={$group['group_id']}";
    
        $res=pg_query($conn, $sql);

        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    }
        
    if(!$err){

        if(isset($group["permissions"])){
            $sql="delete from {$PHORUM['forum_group_xref_table']} where group_id={$group['group_id']}";
    
            $res=pg_query($conn, $sql);
    
            if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    
            foreach($group["permissions"] as $forum_id=>$permission){
                $sql="insert into {$PHORUM['forum_group_xref_table']} set group_id={$group['group_id']}, permission=$permission, forum_id=$forum_id";
                $res=pg_query($conn, $sql);
                if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
                if(!$res) break;
            }
        }
    }

    if($res>0) $ret=true;

    return $ret;

}

/**
 * phorum_db_add_group()
 *
 * @param $group_name
 * @return
 **/
function phorum_db_add_group($group_name)
{
    $PHORUM = $GLOBALS["PHORUM"];
    $conn = phorum_db_postgresql_connect();

    settype($group_id, "integer");

    $sql="insert into {$PHORUM['groups_table']} (name) values ('$group_name')";

    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $group_id = 0;

    if ($res) {
        $group_id = mysql_insert_id($conn);
    }

    return $group_id;
}

/**
* This function returns all moderators for a particular forum
*/
function phorum_db_user_get_moderators($forum_id) {

   $PHORUM = $GLOBALS["PHORUM"];
   $conn = phorum_db_postgresql_connect();
   
   $sql="SELECT DISTINCT(user.email) FROM {$PHORUM['user_table']} as user LEFT JOIN {$PHORUM['user_permissions_table']} as perm ON perm.user_id=user.user_id WHERE (perm.permission > ".PHORUM_USER_ALLOW_MODERATE_MESSAGES." AND (perm.permission & ".PHORUM_USER_ALLOW_MODERATE_MESSAGES." > 0) AND perm.forum_id=$forum_id) OR user.admin=1";
   
   
   $res = pg_query($conn, $sql);
   
   if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");   
   
   $userinfo=array(); 
     
   while ($row = pg_fetch_row($res, 0)){
         $userinfo[]=$row[0];
   }
   
   return $userinfo;
}

/**
 * This function executes a query to select data about a user including
 * his permission data and returns that in an array.
 */

function phorum_db_user_get($user_id)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    if(is_array($user_id)){
        $user_ids=implode(",", $user_id);
    } else {
        $user_ids=(int)$user_id;
    }

    $users = array();

    $sql = "select * from {$PHORUM['user_table']} where user_id in ($user_ids)";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    if (pg_numrows($res)){
        while($rec=pg_fetch_array($res)){
            $users[$rec["user_id"]] = $rec;
        }

        // get the users' permissions
        $sql = "select * from {$PHORUM['user_permissions_table']} where user_id in ($user_ids)";
    
        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    
        while ($row = pg_fetch_array($res)){
            $users[$row["user_id"]]["forum_permissions"][$row["forum_id"]] = $row["permission"];
        }
    
        // get the users' groups and forum permissions through those groups
        $sql = "select user_id, {$PHORUM['user_group_xref_table']}.group_id, forum_id, permission from {$PHORUM['user_group_xref_table']} left join {$PHORUM['forum_group_xref_table']} using (group_id) where user_id in ($user_ids)";

        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
        
        while ($row = pg_fetch_array($res)){
            $users[$row["user_id"]]["groups"][$row["group_id"]] = $row["group_id"];
            if(!empty($row["forum_id"])){
                $users[$row["user_id"]]["group_permissions"][$row["forum_id"]] = $row["permission"];
            }
        }

    }
            
    if(is_array($user_id)){
        return $users;
    } else {
        return $users[$user_id];
    }    

}

/**
 * This function executes a query to select data about a user including
 * his permission data and returns that in an array.
 */

function phorum_db_user_check_pass($username, $password, $temp_password=false){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $username = pg_escape_string($username);

    $md5pass = pg_escape_string($password);

    $pass_field = ($temp_password) ? "password_temp" : "password";

    $sql = "select user_id from {$PHORUM['user_table']} where username='$username' and $pass_field='$password'";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    return ($res && pg_numrows($res)) ? pg_fetch_result($res, 0, "user_id") : 0;
}

/**
 * This function executes a query to check for the given field in the
 * user tableusername and return the user_id of the user it matches or 0
 * if no match is found.
 * 
 * The parameters can be arrays.  If they are, all must be passed and all
 * must have the same number of values.
 */

function phorum_db_user_check_field($field, $value, $operator="="){
    $PHORUM = $GLOBALS["PHORUM"];

    $ret = 0;

    $conn = phorum_db_postgresql_connect();

    if(!is_array($field)){
        $field=array($field);
    }

    if(!is_array($value)){
        $value=array($value);
    }

    if(!is_array($operator)){
        $operator=array($operator);
    }

    foreach($field as $key=>$name){
        $value[$key] = pg_escape_string($value[$key]);
        $clauses[]="$name $operator[$key] '$value[$key]'";
    }

    $sql = "select user_id from {$PHORUM['user_table']} where ".implode(" and ", $clauses);

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    if ($res && pg_numrows($res)){
        $ret = pg_fetch_result($res, 0, "user_id");
    }

    return $ret;
}


/**
 * This function executes a query to add the given user data to the
 * database and returns the userid or 0
 */

function phorum_db_user_add($userdata){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    if (!empty($userdata["permissions"])){
        $perms = $userdata["permissions"];
        unset($userdata["permissions"]);
    }

    $sql = "insert into {$PHORUM['user_table']} set ";

    $values = array();

    foreach($userdata as $key => $value){
        if (!is_numeric($value)){
            $value = pg_escape_string($value);
            $values[] = "$key='$value'";
        }else{
            $values[] = "$key=$value";
        }
    }

    $sql .= implode(", ", $values);

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $user_id = 0;
    if ($res){
        $user_id = mysql_insert_id($conn);
    }

    if ($res && isset($perms)){
        foreach($perms as $fid => $p){
            $sql = "insert into {$PHORUM['user_permissions_table']} set user_id=$user_id, forum_id=$fid, permission=$p";
            $res = pg_query($conn, $sql);
            if ($err = pg_last_error()){
                phorum_db_pg_last_error("$err: $sql");
                break;
            }
        }
    }

    return $user_id;
}


/**
 * This function executes a query to update the given user data in the
 * database and returns the true or false
 */
function phorum_db_user_save($userdata){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    if(isset($userdata["permissions"])){
        unset($userdata["permissions"]);
    }

    if (isset($userdata["forum_permissions"])){
        $forum_perms = $userdata["forum_permissions"];
        unset($userdata["forum_permissions"]);
    }

    if (isset($userdata["groups"])){
        $groups = $userdata["groups"];
        unset($userdata["groups"]);
        unset($userdata["group_permissions"]);
    }

    $user_id = $userdata["user_id"];
    unset($userdata["user_id"]);

    if(count($userdata)){

        $sql = "update {$PHORUM['user_table']} set ";

        $values = array();

        foreach($userdata as $key => $value){
            if (!is_numeric($value)){
                $value = pg_escape_string($value);
                $values[] = "$key='$value'";
            }else{
                $values[] = "$key=$value";
            }
        }

        $sql .= implode(", ", $values);

        $sql .= " where user_id=$user_id";

        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    }

    if (isset($forum_perms)){

        $sql = "delete from {$PHORUM['user_permissions_table']} where user_id = $user_id";
        $res=pg_query($conn, $sql);

        foreach($forum_perms as $fid=>$perms){
            $sql = "insert into {$PHORUM['user_permissions_table']} set user_id=$user_id, forum_id=$fid, permission=$perms";
            $res = pg_query($conn, $sql);
            if ($err = pg_last_error()){
                phorum_db_pg_last_error("$err: $sql");
            }
        }
    }

    if(isset($groups)){
        $sql = "delete from {$PHORUM['user_group_xref_table']} where user_id = $user_id";
        $res=pg_query($conn, $sql);

        foreach($groups as $group_id){
            $sql = "insert into {$PHORUM['user_group_xref_table']} set user_id=$user_id, group_id=$group_id";
            pg_query($conn, $sql);
            if ($err = pg_last_error()){
                phorum_db_pg_last_error("$err: $sql");
                break;
            }
        }

    }

    return (bool)$res;
}

/**
 * This function executes a query to subscribe a user to a forum/thread.
 */

function phorum_db_user_subscribe($user_id, $forum_id, $thread, $type)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $sql = "replace into {$PHORUM['subscribers_table']} set user_id=$user_id, forum_id=$forum_id, sub_type=$type, thread=$thread";

    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    return (bool)$res;
}

/**
  * This function increases the post-counter for a user by one
  */
function phorum_db_user_addpost() {
        
        $conn = phorum_db_postgresql_connect();
        
        $sql="UPDATE ".$GLOBALS['PHORUM']['user_table']." SET posts=posts+1 WHERE user_id = ".$GLOBALS['PHORUM']['user']['user_id'];
        $res=pg_query($conn, $sql);
        
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
        
        return (bool)$res;
}

/**
 * This function executes a query to unsubscribe a user to a forum/thread.
 */

function phorum_db_user_unsubscribe($user_id, $forum_id, $thread, $type)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $sql = "DELETE FROM {$PHORUM['subscribers_table']} WHERE user_id=$user_id AND forum_id=$forum_id AND sub_type=$type AND thread=$thread";

    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    return (bool)$res;
}

/**
 * This function executes a query to select data about a user including
 * his permission data and returns that in an array.
 */

function phorum_db_search_users($search)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $users = array();

    $search = trim($search);

    if(!empty($search)){
        $sql = "select user_id, username, email from {$PHORUM['user_table']} where username like '%$search%' or email like '%$search%'order by username";

        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

        if (pg_numrows($res)){
            while ($user = pg_fetch_array($res)){
                $users[$user["user_id"]] = $user;
            }
        }
    }

    return $users;
}


/**
 * This function gets the users that await approval
 */

function phorum_db_user_get_unapproved()
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $sql="select user_id, username, email from {$PHORUM['user_table']} where active in(".PHORUM_USER_PENDING_BOTH.", ".PHORUM_USER_PENDING_MOD.") order by username";
    $res=pg_query($conn, $sql);
    
    if ($err = pg_last_error()){
        phorum_db_pg_last_error("$err: $sql");
    }
        
    $users=array();
    if($res){
        while($rec=pg_fetch_array($res)){
            $users[$rec["user_id"]]=$rec;
        }
    }
    
    return $users;

}


/**
 * This function gets the users file list
 */

function phorum_db_get_user_file_list($user_id)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $files=array();

    $sql="select file_id, filename, filesize, add_datetime from {$PHORUM['files_table']} where user_id=$user_id";
    
    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()){
        phorum_db_pg_last_error("$err: $sql");
    }
        
    if($res){
        while($rec=pg_fetch_array($res)){
            $files[$rec["file_id"]]=$rec;
        }
    }
    
    return $files;
}


/**
 * This function gets the message's file list
 */

function phorum_db_get_message_file_list($message_id)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $files=array();

    $sql="select file_id, filename, filesize, add_datetime from {$PHORUM['files_table']} where message_id=$message_id";
    
    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()){
        phorum_db_pg_last_error("$err: $sql");
    }
        
    if($res){
        while($rec=pg_fetch_array($res)){
            $files[$rec["file_id"]]=$rec;
        }
    }
    
    return $files;
}


/**
 * This function saves a file to the db
 */

function phorum_db_file_get($file_id)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $file=array();
    
    $sql="select * from {$PHORUM['files_table']} where file_id=$file_id";

    $res = pg_query($conn, $sql);
        
    if ($err = pg_last_error()){
        phorum_db_pg_last_error("$err: $sql");
    }

    if($res){
        $file=pg_fetch_array($res);
    }

    return $file;
}


/**
 * This function saves a file to the db
 */

function phorum_db_file_save($user_id, $filename, $filesize, $buffer, $message_id=0)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $file_id=0;

    settype($user_id, "int");
    settype($message_id, "int");
    $filename=addslashes($filename);
    settype($filesize, "int");

    $sql="insert into {$PHORUM['files_table']} set user_id=$user_id, message_id=$message_id, filename='$filename', filesize=$filesize, file_data='$buffer', add_datetime=".time();

    $res = pg_query($conn, $sql);
        
    if ($err = pg_last_error()){
        phorum_db_pg_last_error("$err: $sql");
    }

    if($res){
        $file_id=mysql_insert_id($conn);
    }

    return $file_id;
}


/**
 * This function saves a file to the db
 */

function phorum_db_file_delete($file_id)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $file=array();
    
    $sql="delete from {$PHORUM['files_table']} where file_id=$file_id";

    $res = pg_query($conn, $sql);
        
    if ($err = pg_last_error()){
        phorum_db_pg_last_error("$err: $sql");
    }

    return $res;
}


/**
 * This function reads the current total size of all files for a user
 */

function phorum_db_get_user_filesize_total($user_id)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $total=0;

    $sql="select sum(filesize) as total from {$PHORUM['files_table']} where user_id=$user_id";

    $res = pg_query($conn, $sql);
        
    if ($err = pg_last_error()){
        phorum_db_pg_last_error("$err: $sql");
    }

    if($res){
        $total=pg_fetch_result($res, 0,"total");
    }

    return $total;

}


/**
 * This function reads the newflags from the database
 */

function phorum_db_read_newflags()
{
    $PHORUM = $GLOBALS["PHORUM"];
    $newflags = "";

    $conn = phorum_db_postgresql_connect();

    $sql = "select newflags from " . $PHORUM['user_newflags_table'] . " where user_id=" . $PHORUM['user']['user_id'] . " AND forum_id=" . $PHORUM['forum_id'];

    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()){
        phorum_db_pg_last_error("$err: $sql");
    }elseif (pg_numrows($res) > 0){
        $newflags = pg_fetch_row($res, 0);
        $GLOBALS['PHORUM']['user']['newinfo'] = unserialize($newflags[0]);
    }else{
        $GLOBALS['PHORUM']['user']['newinfo'] = array("min_id" => 0, "min_timestamp" => 0);
    }
    return (bool)$res;
}

/**
 * This function returns the newinfo-array for markallread
 */

function phorum_db_allread_newflags(){
    $PHORUM = $GLOBALS['PHORUM'];
    $conn = phorum_db_postgresql_connect();
    $sql = "select max(message_id),max(modifystamp) from {$PHORUM['message_table']} where forum_id={$PHORUM['forum_id']}";
    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()){
        phorum_db_pg_last_error("$err: $sql");
    }elseif (pg_numrows($res) > 0){
        $row = pg_fetch_row($res, 0);
        $GLOBALS['PHORUM']['user']['newinfo']['min_id'] = $row[0];
        $GLOBALS['PHORUM']['user']['newinfo']['min_timestamp'] = $row[1];
    }else{
        $GLOBALS['PHORUM']['user']['newinfo'] = array();
    }
}

/**
 * This function outputs some nice formatted debugging data (only for dev)
 */

function debug_out($var, $desc = ''){
    if (DEBUG){
        echo "<br>$desc: <pre>";
        print_r($var);
        echo "</pre><br>";
    }
}

/**
 * This function inserts the data to the table for what messages a user has read.
 */

function phorum_db_write_newflags(){
    $PHORUM = $GLOBALS["PHORUM"];
    // compute the array if we have to much newinfos
    $count = sizeof($PHORUM['user']['newinfo']);
    $count -= 2;
    debug_out($PHORUM['user']['newinfo'], 'Newinfo - Anfang');
    if ($count > PHORUM_MAX_NEW_INFO){
        // sort it as required
        reset($PHORUM['user']['newinfo']);
        if (!$PHORUM['threaded_list'] && $PHORUM['float_to_top'])
            asort($PHORUM['user']['newinfo']);
        else
            ksort($PHORUM['user']['newinfo']);

        debug_out($PHORUM['user']['newinfo'], 'Newinfo - sortiert');

        reset($PHORUM['user']['newinfo']);
        // now remove the number of newflags to get below the max-number
        while ($count > PHORUM_MAX_NEW_INFO){
            $ret = each($PHORUM['user']['newinfo']);
            // in some spurious cases its not an array anymore
            if (!is_array($ret))
                break;

            debug_out($ret, 'bearbeiteter Wert');
            // rescue our string-keys
            if (!is_int($ret['key'])){
                $PHORUM['user']['newinfo'][$ret['key']] = $ret['value'];
            }else{
                $count--;
                unset($PHORUM['user']['newinfo'][$ret['key']]);
                // set our min-keys to the new minimum
                $PHORUM['user']['newinfo']['min_id'] = $ret['key'];
                $PHORUM['user']['newinfo']['min_timestamp'] = $ret['value'];
            }
        }

        debug_out($PHORUM['user']['newinfo'], 'Newinfo - Ende');
    }

    $conn = phorum_db_postgresql_connect();

    $sql = "replace into " . $PHORUM['user_newflags_table'] . " set newflags='" . pg_escape_string(serialize($PHORUM['user']['newinfo'])) . "', user_id=" . $PHORUM['user']['user_id'] . ", forum_id=" . $PHORUM['forum_id'];

    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    return (bool)$res;
}

/**
 * This function executes a query to get the user ids of the users
 * subscribed to a forum/thread.
 */

function phorum_db_get_subscribed_users($forum_id, $thread, $type){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $userignore="";
    if ($PHORUM["DATA"]["LOGGEDIN"])
       $userignore="and b.user_id != {$PHORUM['user']['user_id']}";

    $sql = "select DISTINCT(b.email) from {$PHORUM['subscribers_table']} as a,{$PHORUM['user_table']} as b where a.forum_id=$forum_id and (a.thread=$thread or a.thread=0) and a.sub_type=$type and b.user_id=a.user_id $userignore";

    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

        $arr=array();

    while ($rec = pg_fetch_row($res, 0)){
        $arr[] = $rec[0];
    }

    return $arr;
}

/**
 * This function executes a query to get the subscriptions of a user-id,
 * together with the forum-id and subjects of the threads
 */

function phorum_db_get_message_subscriptions($user_id,$days=2){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $userignore="";
    if ($PHORUM["DATA"]["LOGGEDIN"])
       $userignore="and b.user_id != {$PHORUM['user']['user_id']}";

    if($days > 0) {
         $timestr=" AND (".time()." - b.modifystamp) <= ($days * 86400)";
    } else {
        $timestr="";
    }

    $sql = "select a.thread, a.forum_id, a.sub_type, b.subject,b.datestamp,b.author from {$PHORUM['subscribers_table']} as a,{$PHORUM['message_table']} as b where a.user_id=$user_id and b.message_id=a.thread and (a.sub_type=".PHORUM_SUBSCRIPTION_MESSAGE." or a.sub_type=".PHORUM_SUBSCRIPTION_BOOKMARK.")"."$timestr ORDER BY b.forum_id";

    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $arr=array();
    $forum_ids=array();

    while ($rec = pg_fetch_array($res)){
        $unsub_url=phorum_get_url(PHORUM_CONTROLCENTER_URL, "panel=".PHORUM_CC_SUBSCRIPTION_THREADS, "unsub_id=".$rec['thread'], "unsub_forum=".$rec['forum_id'], "unsub_type=".$rec['sub_type']);
        $rec['unsubscribe_url']=$unsub_url;
        $arr[] = $rec;
        $forum_ids[]=$rec['forum_id'];
    }
    $arr['forum_ids']=$forum_ids;

    return $arr;
}

/**
 * This function executes a query to find out if a user is subscribed to a thread
 */

function phorum_db_get_if_subscribed($forum_id, $thread, $user_id, $type=PHORUM_SUBSCRIPTION_MESSAGE){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $sql = "select user_id from {$PHORUM['subscribers_table']} where forum_id=$forum_id and thread=$thread and user_id=$user_id and sub_type=$type";

    $res = pg_query($conn, $sql);

    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    if (pg_numrows($res) > 0){
        $retval = true;
    }else{
        $retval = false;
    }

    return $retval;
}


/**
 * This function retrieves the banlists for the current forum 
 */
 
function phorum_db_get_banlists() {
    $PHORUM = $GLOBALS["PHORUM"];
    
    $retarr = array();
    $forumstr = "";
    
    $conn = phorum_db_postgresql_connect();
    
    if(isset($PHORUM['forum_id']) && !empty($PHORUM['forum_id']))
        $forumstr = "WHERE forum_id = {$PHORUM['forum_id']} OR forum_id = 0";
        
    
    
    $sql = "SELECT * FROM {$PHORUM['banlist_table']} $forumstr";
    
    $res = pg_query($conn, $sql);
    
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    
    if (pg_numrows($res) > 0){
        while($row = pg_fetch_array($res)) {
            $retarr[$row['type']][$row['id']]=array('pcre'=>$row['pcre'],'string'=>$row['string'],'forum_id'=>$row['forum_id']);
        }
    }
    return $retarr;
}


/**
 * This function retrieves one item from the banlists
 */
 
function phorum_db_get_banitem($banid) {
    $PHORUM = $GLOBALS["PHORUM"];
    
    $retarr = array();
    
    $conn = phorum_db_postgresql_connect();
    
    $sql = "SELECT * FROM {$PHORUM['banlist_table']} WHERE id = $banid";
    
    $res = pg_query($conn, $sql);
    
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    
    if (pg_numrows($res) > 0){
        while($row = pg_fetch_array($res)) {
            $retarr=array('pcre'=>$row['pcre'],'string'=>$row['string'],'forumid'=>$row['forum_id'],'type'=>$row['type']);
        }
    }
    return $retarr;
}


/**
 * This function deletes one item from the banlists
 */
 
function phorum_db_del_banitem($banid) {
    $PHORUM = $GLOBALS["PHORUM"];
    
    $conn = phorum_db_postgresql_connect();
    
    $sql = "DELETE FROM {$PHORUM['banlist_table']} WHERE id = $banid";
    
    $res = pg_query($conn, $sql);
    
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    
    if(pg_affected_rows($conn) > 0) {
        return true;   
    } else {
        return false;
    }
}


/**
 * This function adds or modifies a banlist-entry
 */
 
function phorum_db_mod_banlists($type,$pcre,$string,$forum_id,$id=0) {
    $PHORUM = $GLOBALS["PHORUM"];
    
    $retarr = array();
    
    $conn = phorum_db_postgresql_connect();
    
    if($id > 0) { // modifying an entry
        $sql = "UPDATE {$PHORUM['banlist_table']} SET forum_id = $forum_id, type = $type, pcre = $pcre, string = '".pg_escape_string($string)."' where id = $id";
    } else { // adding an entry
        $sql = "INSERT INTO {$PHORUM['banlist_table']} (forum_id,type,pcre,string) VALUES($forum_id,$type,$pcre,'".pg_escape_string($string)."')";
    }

    $res = pg_query($conn, $sql);
    
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    
    if(pg_affected_rows($conn) > 0) {
        return true;   
    } else {
        return false;
    }
}
 


/**
 * This function retrives private messages
 */

function phorum_db_get_private_messages($user_id, $type)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    if($type!="from" && $type!="to"){
        trigger_error("\$type must be either `to` or `from` in function phorum_db_get_private_messages()", E_USER_WARNING);
        return 0;
    }
    
    $field1=$type."_user_id";
    $field2=$type."_del_flag";
    
    $retarr=array();

    $sql="select * from {$PHORUM['private_message_table']} where $field1=$user_id and $field2=0 order by $field1 desc, $field2 desc, datestamp desc";
    $res = pg_query($conn, $sql);
    
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    
    if (pg_numrows($res) > 0){
        while($row = pg_fetch_array($res)) {
            $retarr[]=$row;
        }
    }

    return $retarr;
}


/**
 * This function retrives private messages
 */

function phorum_db_get_private_message($pm_id)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    settype($pm_id, "int");
    
    $retarr=array();

    $sql="select * from {$PHORUM['private_message_table']} where private_message_id=$pm_id";
    $res = pg_query($conn, $sql);
    
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
    
    if (pg_numrows($res) > 0){
        $retarr = pg_fetch_array($res);
    }

    return $retarr;
}


/**
 * This function inserts a private message
 */

function phorum_db_put_private_messages($to_username, $to_user_id, $subject, $message, $keep)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $from_delete_flag = (empty($keep)) ? 1 : 0;

    $sql="insert into {$PHORUM['private_message_table']} set
            from_username  = '".pg_escape_string($PHORUM['user']['username'])."',
            to_username    = '".pg_escape_string($to_username)."',
            from_user_id   = '{$PHORUM['user']['user_id']}',
            to_user_id     = '$to_user_id',
            subject        = '".pg_escape_string($subject)."',
            message        = '".pg_escape_string($message)."',
            datestamp      = '".time()."',
            from_del_flag  = '$from_delete_flag'";

    $res = pg_query($conn, $sql);
        
    if ($err = pg_last_error()){
        phorum_db_pg_last_error("$err: $sql");
    }

    return $res;
}


/**
 * This function updates the flags in a pm
 */

function phorum_db_update_private_message($pm_id, $flag, $value)
{
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    if($flag!="read_flag" && $flag!="return_flag" && $flag!="reply_flag" && $flag!="to_del_flag" && $flag!="from_del_flag"){
        trigger_error("Invalid value for \$flag in function phorum_db_update_private_message()", E_USER_WARNING);
        return 0;
    }

    settype($value, "int");
    settype($pm_id, "int");
    
    $sql="update {$PHORUM['private_message_table']} set $flag=$value where private_message_id=$pm_id";

    $res = pg_query($conn, $sql);
        
    if ($err = pg_last_error()){
        phorum_db_pg_last_error("$err: $sql");
    }

    if($res>0 && ($flag=="to_del_flag" || $flag=="from_del_flag")){
        // clear any messages where both delete flags are set
        $sql="delete from {$PHORUM['private_message_table']} where to_del_flag=1 and from_del_flag=1";
        pg_query($conn, $sql);
    }

    return $res;
}


/**
 * This function creates the tables needed in the database.
 */

function phorum_db_create_tables()
{
    $PHORUM     = $GLOBALS["PHORUM"];
    $PHORUM_IDX = $GLOBALS["PHORUM_IDX"];

    $conn = phorum_db_postgresql_connect();
	phorum_db_pg_begin($conn);

    $retmsg = "";

    $queries = array(

        // create tables
        "CREATE TABLE {$PHORUM['forums_table']} (             forum_id               serial, 
                                                              name                   varchar(50)  NOT NULL default '',
                                                              active                 smallint     NOT NULL default 0,
                                                              description            varchar(255) NOT NULL default '',
                                                              template               varchar(50)  NOT NULL default '',
                                                              folder_flag            smallint     NOT NULL default 0,
                                                              parent_id              int          NOT NULL default 0,
                                                              list_length            int          NOT NULL default 0,
                                                              moderation             int          NOT NULL default 0,
                                                              email_outgoing_address varchar(50)  NOT NULL default '',
                                                              email_incoming_address varchar(50)  NOT NULL default '',
                                                              email_subject_tag      varchar(20)  NOT NULL default '',
                                                              threaded_list          smallint     NOT NULL default 0,
                                                              threaded_read          smallint     NOT NULL default 0,
                                                              float_to_top           smallint     NOT NULL default 0,
                                                              check_duplicate        smallint     NOT NULL default 0,
                                                              allow_attachment_types varchar(100) NOT NULL default '',
                                                              max_attachment_size    int          NOT NULL default 0,
                                                              max_attachments        int          NOT NULL default 0,
                                                              pub_perms              int          NOT NULL default 0,
                                                              reg_perms              int          NOT NULL default 0,
                                                              display_ip_address     smallint     NOT NULL default 1,
                                                              allow_email_notify     smallint     NOT NULL default 1,
                                                              language               varchar(100) NOT NULL default 'english',
                                                              email_moderators       smallint     NOT NULL default 0,
                                                              message_count          int          NOT NULL default 0,
                                                              thread_count           int          NOT NULL default 0,
                                                              last_post_time         int          NOT NULL default 0,
                                                              display_order          int          NOT NULL default 0,
                                                              read_length            int          NOT NULL default 0,
                                                              edit_post              smallint     NOT NULL default 1,
                                                              template_settings      text         NOT NULL,
                                                              registration_control   smallint     NOT NULL default 0,
                                                              PRIMARY KEY  (forum_id))",

        "CREATE INDEX {$PHORUM_IDX['forums_table_name']}     on {$PHORUM['forums_table']} (name)",
        "CREATE INDEX {$PHORUM_IDX['forums_table_active']}   on {$PHORUM['forums_table']} (active, parent_id)",
        "CREATE INDEX {$PHORUM_IDX['forums_table_group_id']} on {$PHORUM['forums_table']} (parent_id)",

        "CREATE TABLE {$PHORUM['message_table']} (            message_id             serial,
                                                              forum_id               int          NOT NULL default 0,
                                                              thread                 int          NOT NULL default 0,
                                                              parent_id              int          NOT NULL default 0,
                                                              author                 varchar(37)  NOT NULL default '',
                                                              subject                varchar(255) NOT NULL default '',
                                                              body                   text         NOT NULL,
                                                              email                  varchar(200) NOT NULL default '',
                                                              ip                     varchar(50)  NOT NULL default '',
                                                              status                 smallint     NOT NULL default 2,
                                                              msgid                  varchar(100) NOT NULL default '',
                                                              modifystamp            int          NOT NULL default 0,
                                                              user_id                int          NOT NULL default 0,
                                                              thread_count           int          NOT NULL default 0,
                                                              moderator_post         smallint     NOT NULL default 0,
                                                              sort                   smallint     NOT NULL default 2,
                                                              datestamp              int          NOT NULL default 0,
                                                              meta                   text         NOT NULL,
                                                              PRIMARY KEY         (message_id) )", 

        "CREATE INDEX {$PHORUM_IDX['message_table_message_id']}       on {$PHORUM['message_table']} (message_id,   forum_id)",
        "CREATE INDEX {$PHORUM_IDX['message_table_thread_message']}   on {$PHORUM['message_table']} (thread,       message_id)",
        "CREATE INDEX {$PHORUM_IDX['message_table_thread_forum']}     on {$PHORUM['message_table']} (thread,       forum_id)",
        "CREATE INDEX {$PHORUM_IDX['message_table_list_page_flat']}   on {$PHORUM['message_table']} (thread,       forum_id, status, parent_id, sort)",
        "CREATE INDEX {$PHORUM_IDX['message_table_special_threads']}  on {$PHORUM['message_table']} (sort,         forum_id)",
        "CREATE INDEX {$PHORUM_IDX['message_table_list_page_float']}  on {$PHORUM['message_table']} (modifystamp,  forum_id, status, parent_id, sort)",

        "CREATE TABLE {$PHORUM['settings_table']} (           name                   varchar(255) NOT NULL default '',
                                                              type                   char(1)      NOT NULL default 'V' check (type in ('V','S')),
                                                              data                   text         NOT NULL,
                                                              PRIMARY KEY (name))",

        "CREATE TABLE {$PHORUM['subscribers_table']} (        user_id                int          NOT NULL default 0,
                                                              forum_id               int          NOT NULL default 0,
                                                              sub_type               int          NOT NULL default 0,
                                                              thread                 int          NOT NULL default 0,
                                                              PRIMARY KEY  (user_id,  forum_id, thread))",

        "CREATE INDEX {$PHORUM_IDX['subscribers_table_forum_id']}  on {$PHORUM['subscribers_table']} (forum_id, thread, sub_type)",


        "CREATE TABLE {$PHORUM['user_permissions_table']} (   user_id                int          NOT NULL default 0,
                                                              forum_id               int          NOT NULL default 0,
                                                              permission             int          NOT NULL default 0,
                                                              PRIMARY KEY  (user_id,  forum_id))",

        "CREATE INDEX {$PHORUM_IDX['user_permissions_table_forum_id']}  on {$PHORUM['user_permissions_table']} (forum_id, permission)",

        "CREATE TABLE {$PHORUM['user_table']} (               user_id                serial,
                                                              username               varchar(50)  NOT NULL default '',
                                                              password               varchar(50)  NOT NULL default '',
                                                              password_temp          varchar(50)  NOT NULL default '',
                                                              email                  varchar(50)  NOT NULL default '',
                                                              hide_email             smallint     NOT NULL default 0,
                                                              active                 smallint     NOT NULL default 0,
                                                              user_data              text         NOT NULL,
                                                              signature              text         NOT NULL,
                                                              threaded_list          smallint     NOT NULL default 0,
                                                              threaded_read          smallint     NOT NULL default 0,
                                                              posts                  int          NOT NULL default 0,
                                                              admin                  smallint     NOT NULL default 0,
                                                              PRIMARY KEY (user_id))",

        "CREATE UNIQUE INDEX {$PHORUM_IDX['user_table_username']} on {$PHORUM['user_table']} (username)",
        "CREATE INDEX        {$PHORUM_IDX['user_table_active']}   on {$PHORUM['user_table']} (active)",
        "CREATE INDEX        {$PHORUM_IDX['user_table_userpass']} on {$PHORUM['user_table']} (username, password)",

        "CREATE TABLE {$PHORUM['user_newflags_table']} (      user_id                int          NOT NULL default 0,
                                                              forum_id               int          NOT NULL default 0,
                                                              newflags               text         NOT NULL,
                                                              PRIMARY KEY (user_id, forum_id))",

        "CREATE TABLE {$PHORUM['groups_table']} (             group_id               serial,
                                                              name                   varchar(255) NOT NULL default 0,
                                                              PRIMARY KEY (group_id) )",

        "CREATE TABLE {$PHORUM['forum_group_xref_table']} (   forum_id               int          NOT NULL default 0,
                                                              group_id               int          NOT NULL default 0,
                                                              permission             int          NOT NULL default 0,
                                                              PRIMARY KEY (forum_id, group_id) )",

        "CREATE TABLE {$PHORUM['user_group_xref_table']} (    user_id                int          NOT NULL default 0,
                                                              group_id               int          NOT NULL default 0,
                                                              PRIMARY KEY (user_id, group_id) )",

        "CREATE TABLE {$PHORUM['files_table']} (              file_id                serial,
                                                              user_id                int          NOT NULL default 0,
                                                              filename               varchar(255) NOT NULL default '',
                                                              filesize               int          NOT NULL default 0,
                                                              file_data              text         NOT NULL,
                                                              add_datetime           int          NOT NULL default 0,
                                                              message_id             int          NOT NULL default 0,
                                                              PRIMARY KEY      (file_id))",

        "CREATE INDEX        {$PHORUM_IDX['files_table_add_datetime']} on {$PHORUM['files_table']} (add_datetime)",
        "CREATE INDEX        {$PHORUM_IDX['files_table_message_id']}   on {$PHORUM['files_table']} (message_id)",

        "CREATE TABLE {$PHORUM['message_files_xref_table']} ( message_id             int          NOT NULL default 0,
                                                              file_id                int          NOT NULL default 0,
                                                              PRIMARY KEY (message_id, file_id))",

        "CREATE TABLE {$PHORUM['banlist_table']} (            id                     serial,
                                                              forum_id               int          NOT NULL default 0,
                                                              type                   smallint     NOT NULL default 0,
                                                              pcre                   smallint     NOT NULL default 0,
                                                              string                 varchar(255) NOT NULL default '',
                                                              PRIMARY KEY  (id))",

        "CREATE INDEX {$PHORUM_IDX['banlist_table_forum_id']}   on {$PHORUM['banlist_table']} (forum_id)",

        "CREATE TABLE {$PHORUM['private_message_table']} (    private_message_id     serial,
                                                              from_username          varchar(50)  NOT NULL default '',
                                                              to_username            varchar(50)  NOT NULL default '',
                                                              from_user_id           int          NOT NULL default 0,
                                                              to_user_id             int          NOT NULL default 0,
                                                              subject                varchar(100) NOT NULL default '',
                                                              message                text         NOT NULL,
                                                              datestamp              int          NOT NULL default 0,
                                                              read_flag              smallint     NOT NULL default 0,
                                                              reply_flag             smallint     NOT NULL default 0,
                                                              to_del_flag            smallint     NOT NULL default 0,
                                                              from_del_flag          smallint     NOT NULL default 0,
                                                              PRIMARY KEY      (private_message_id))",

        "CREATE INDEX {$PHORUM_IDX['private_message_table_to_user_id']}   on {$PHORUM['private_message_table']} (to_user_id,   to_del_flag,   datestamp)",
        "CREATE INDEX {$PHORUM_IDX['private_message_table_from_user_id']} on {$PHORUM['private_message_table']} (from_user_id, from_del_flag, datestamp)",
        "CREATE INDEX {$PHORUM_IDX['private_message_table_to_del_flag']}  on {$PHORUM['private_message_table']} (to_del_flag,  from_del_flag)",

    );

    foreach($queries as $sql){
echo "<pre>query='$sql'</pre>";
        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()){
            $retmsg = "$err<br />";
            phorum_db_pg_last_error("$err: $sql");
            break;
        }
    }


    if(empty($err)){

        $tmp_dir = (substr(__FILE__, 0, 1)=="/") ? "/tmp" : "C:\\Windows\\Temp";

        // set initial settings
        $settings=array(
            "title"                   => "Phorum 5",
            "cache"                   => "/tmp",
            "session_timeout"         => "30",
            "session_path"            => "/",
            "session_domain"          => "",
            "cache_users"             => "0",
            "register_email_confirm"  => "0",
            "default_template"        => "default",
            "default_language"        => "english",
            "use_cookies"             => "1",
            "use_bcc"                 => "1",
            "internal_version"        => "" . PHORUMINTERNAL . "",
            "PROFILE_FIELDS"          => array("real_name")
        );

#        phorum_db_update_settings($settings);
        

        // create a test forum
        $forum=array(
                  "name"=>'Test Forum',
                  "active"=>0,
                  "description"=>'This is a test forum.  Feel free to delete it or edit after installation.',
                  "template"=>'default',
                  "folder_flag"=>0,
                  "parent_id"=>0,
                  "list_length"=>30,
                  "read_length"=>20,
                  "moderation"=>0,
                  "threaded_list"=>0,
                  "threaded_read"=>0,				  
                  "float_to_top"=>1,
                  "display_ip_address"=>0,
                  "allow_email_notify"=>1,
                  "language"=>'english',
                  "email_moderators"=>0,
                  "display_order"=>0,
                  "edit_post"=>1,
                  "pub_perms" =>  1,
                  "reg_perms" =>  15
              );

#        $GLOBALS["PHORUM"]['forum_id']=phorum_db_add_forum($forum);
        
        // create a test post
        $message=array(
                    "forum_id" => $GLOBALS['PHORUM']["forum_id"],
                    "thread" => 0,
                    "parent_id" => 0,
                    "author" => 'Phorum Installer',
                    "subject" => 'Test Message',
                    "email" => '',
                    "ip" => '127.0.0.1',
                    "user_id" => 0,
                    "moderator_post" => 0,
                    "status" => PHORUM_STATUS_APPROVED,
                    "sort" => PHORUM_SORT_DEFAULT,
                    "msgid" => '',
                    "body" => "This is a test message.  You can delete it after install using the admin.\n\nPhorum 5 Team"
                 );

        phorum_db_post_message($message);

        include_once ("./include/thread_info.php");

        phorum_update_thread_info($message["thread"]);

        phorum_db_update_forum_stats(true);

    }

    return $retmsg;
}

/**
 * This function goes through an array of queries and executes them
 */

function phorum_db_run_queries($queries){
    $PHORUM = $GLOBALS["PHORUM"];

    $conn = phorum_db_postgresql_connect();

    $retmsg = "";

    foreach($queries as $sql){
        $res = pg_query($conn, $sql);
        if ($err = pg_last_error()){
            // skip duplicate column name errors
            if(!stristr($err, "duplicate column")){
                $retmsg.= "$err<br />";
                phorum_db_pg_last_error("$err: $sql");
            }
        }
    }

    return $retmsg;
}

/**
 * This function checks that a database connection can be made.
 */

function phorum_db_check_connection(){
    $conn = phorum_db_postgresql_connect();

    return ($conn > 0) ? true : false;
}

/**
 * handy little connection function.  This allows us to not connect to the
 * server until a query is actually run.
 * NOTE: This is not a required part of abstraction
 */

function phorum_db_postgresql_connect(){
    $PHORUM = $GLOBALS["PHORUM"];

    static $conn;

	$connectstring = '';
    if (empty($conn)) {
		#
		# do not specify the host if it is either empty or localhost
		#
		if (strtolower($PHORUM["DBCONFIG"]["server"]) != 'localhost' &&
		               $PHORUM["DBCONFIG"]["server"]  <> '') {
			$connectstring .= " host="      . $PHORUM["DBCONFIG"]["server"];
		}
		$connectstring .= " dbname="   . $PHORUM["DBCONFIG"]["name"]     . 
    	                  " user="     . $PHORUM["DBCONFIG"]["user"]     .
		                  " password=" . $PHORUM["DBCONFIG"]["password"];

echo "\$connectstring=<pre>$connectstring</pre>";
        $conn = pg_connect($connectstring);
    }
    return $conn;
}

/**
 * error handling function
 * NOTE: This is not a required part of abstraction
 */

function phorum_db_pg_last_error($err){
    if (!defined("PHORUM_ADMIN")){
        echo htmlspecialchars($err);
        exit();
    }else{
        echo "<!-- $err -->";
    }
}

function phorum_db_pg_insertid($conn, $sequence_name) {
    $sql = "select currval($sequence_name)";

    $res = pg_query($conn, $sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");

    $sequence_value = pg_fetch_result($res, 0, 0);

    return $sequence_value;
}

function phorum_db_pg_begin($conn) {
	$sql = "begin";
	$err = pg_query($sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
}

function phorum_db_pg_commit($conn) {
	$sql = "commit";
	$err = pg_query($sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
}

function phorum_db_pg_rollback($conn) {
	$sql = "rollback";
	$err = pg_query($sql);
    if ($err = pg_last_error()) phorum_db_pg_last_error("$err: $sql");
}
?>
